@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 
@@ -27,8 +27,6 @@
 #include "pangoxft-private.h"
 #include "pangofc-private.h"
 
-PangoXftWarningHistory _pango_xft_warning_history = { FALSE };
-
 #define PANGO_XFT_FONT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_XFT_FONT, PangoXftFontClass))
 #define PANGO_XFT_IS_FONT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_XFT_FONT))
 #define PANGO_XFT_FONT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_XFT_FONT, PangoXftFontClass))
@@ -67,7 +65,7 @@ pango_xft_font_class_init (PangoXftFontClass *class)
   PangoFcFontClass *fc_font_class = PANGO_FC_FONT_CLASS (class);
 
   object_class->finalize = pango_xft_font_finalize;
-  
+
   font_class->get_glyph_extents = pango_xft_font_get_glyph_extents;
 
   fc_font_class->lock_face = pango_xft_font_real_lock_face;
@@ -101,7 +99,7 @@ _pango_xft_font_new (PangoXftFontMap *xftfontmap,
    * want integer line spacing, underline positions, etc
    */
   PANGO_FC_FONT (xfont)->is_hinted = TRUE;
-  
+
   xfont->xft_font = NULL;
 
   return xfont;
@@ -110,11 +108,11 @@ _pango_xft_font_new (PangoXftFontMap *xftfontmap,
 /**
  * _pango_xft_font_get_mini_font:
  * @xfont: a #PangoXftFont
- * 
+ *
  * Gets the font used for drawing the digits in the
  * missing-character hex squares
- * 
- * Return value: the PangoFont used for the digits; this
+ *
+ * Return value: the #PangoFont used for the digits; this
  *  value is associated with the main font and will be freed
  *  along with the main font.
  **/
@@ -137,30 +135,30 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
       XGlyphInfo extents;
       XftFont *mini_xft;
       int new_size;
-      
+
       _pango_xft_font_map_get_info (fcfont->fontmap, &display, &screen);
 
-      context = pango_xft_get_context (display, screen);
+      context = pango_font_map_create_context (pango_xft_get_font_map (display, screen));
       pango_context_set_language (context, pango_language_from_string ("en"));
 
       pango_font_description_set_family_static (desc, "monospace");
 
       new_size = pango_font_description_get_size (fcfont->description) / 2;
-      
+
       if (pango_font_description_get_size_is_absolute (fcfont->description))
 	pango_font_description_set_absolute_size (desc, new_size);
       else
 	pango_font_description_set_size (desc, new_size);
 
       xfont->mini_font = pango_font_map_load_font (fcfont->fontmap, context, desc);
-      if (!xfont->mini_font)
-        return NULL;
-
       pango_font_description_free (desc);
       g_object_unref (context);
-      
+
+      if (!xfont->mini_font)
+	return NULL;
+
       mini_xft = xft_font_get_font (xfont->mini_font);
-      
+
       for (i = 0 ; i < 16 ; i++)
 	{
 	  char c = i < 10 ? '0' + i : 'A' + i - 10;
@@ -169,10 +167,9 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
 	  height = MAX (height, extents.height);
 	}
 
-      
       xfont->mini_width = PANGO_SCALE * width;
       xfont->mini_height = PANGO_SCALE * height;
-      xfont->mini_pad = PANGO_SCALE * MAX ((int)(2.2 * height + 27) / 28, 1);
+      xfont->mini_pad = PANGO_SCALE * MIN (height / 2, MAX ((int)(2.2 * height + 27) / 28, 1));
     }
 
   return xfont->mini_font;
@@ -190,7 +187,7 @@ pango_xft_font_finalize (GObject *object)
   if (xfont->xft_font)
     {
       Display *display;
-      
+
       _pango_xft_font_map_get_info (fcfont->fontmap, &display, NULL);
       XftFontClose (display, xfont->xft_font);
     }
@@ -210,11 +207,18 @@ get_glyph_extents_missing (PangoXftFont    *xfont,
 {
   PangoFont *font = PANGO_FONT (xfont);
   XftFont *xft_font = xft_font_get_font (font);
+  gunichar ch;
+  gint cols;
   
-  gint cols = (glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 3 : 2;
-  
+  ch = glyph & ~PANGO_GLYPH_UNKNOWN_FLAG;
+
+  if (G_UNLIKELY (glyph == PANGO_GLYPH_INVALID_INPUT || ch > 0x10FFFF))
+    cols = 1;
+  else
+    cols = ch > 0xffff ? 3 : 2;
+
   _pango_xft_font_get_mini_font (xfont);
-  
+
   if (ink_rect)
     {
       ink_rect->x = 0;
@@ -222,7 +226,7 @@ get_glyph_extents_missing (PangoXftFont    *xfont,
       ink_rect->width = xfont->mini_width * cols + xfont->mini_pad * (2 * cols + 1);
       ink_rect->height = xfont->mini_height * 2 + xfont->mini_pad * 5;
     }
-  
+
   if (logical_rect)
     {
       logical_rect->x = 0;
@@ -242,7 +246,7 @@ get_glyph_extents_xft (PangoFcFont      *fcfont,
   XGlyphInfo extents;
   Display *display;
   FT_UInt ft_glyph = glyph;
-  
+
   _pango_xft_font_map_get_info (fcfont->fontmap, &display, NULL);
 
   XftGlyphExtents (display, xft_font, &ft_glyph, 1, &extents);
@@ -254,7 +258,7 @@ get_glyph_extents_xft (PangoFcFont      *fcfont,
       ink_rect->width = extents.width * PANGO_SCALE;
       ink_rect->height = extents.height * PANGO_SCALE;
     }
-  
+
   if (logical_rect)
     {
       logical_rect->x = 0;
@@ -294,7 +298,7 @@ get_glyph_extents_raw (PangoXftFont     *xfont,
   if (!extents)
     {
       extents = g_slice_new (Extents);
-     
+
       pango_fc_font_get_raw_extents (PANGO_FC_FONT (xfont),
 				     FT_LOAD_NO_BITMAP | FT_LOAD_NO_HINTING,
 				     glyph,
@@ -305,7 +309,7 @@ get_glyph_extents_raw (PangoXftFont     *xfont,
 			   GUINT_TO_POINTER (glyph),
 			   extents);
     }
-  
+
   if (ink_rect)
     *ink_rect = extents->ink_rect;
 
@@ -356,17 +360,17 @@ load_fallback_font (PangoXftFont *xfont)
   XftFont *xft_font;
   gboolean size_is_absolute;
   double size;
-  
+
   _pango_xft_font_map_get_info (fcfont->fontmap, &display, &screen);
-  
+
   size_is_absolute = pango_font_description_get_size_is_absolute (fcfont->description);
   size = pango_font_description_get_size (fcfont->description) / PANGO_SCALE;
-      
+
   xft_font = XftFontOpen (display,  screen,
-                          FC_FAMILY, FcTypeString, "sans",
+			  FC_FAMILY, FcTypeString, "sans",
 			  size_is_absolute ? FC_PIXEL_SIZE : FC_SIZE, FcTypeDouble, size,
-                          NULL);
-  
+			  NULL);
+
   if (!xft_font)
     {
       g_warning ("Cannot open fallback font, nothing to do");
@@ -387,7 +391,7 @@ xft_font_get_font (PangoFont *font)
   xfont = (PangoXftFont *)font;
   fcfont = (PangoFcFont *)font;
 
-  if (xfont->xft_font == NULL)
+  if (G_UNLIKELY (xfont->xft_font == NULL))
     {
       FcPattern *pattern = FcPatternDuplicate (fcfont->font_pattern);
       FcPatternDel (pattern, FC_SPACING);
@@ -399,12 +403,12 @@ xft_font_get_font (PangoFont *font)
 	{
 	  gchar *name = pango_font_description_to_string (fcfont->description);
 	  g_warning ("Cannot open font file for font %s", name);
-  	  g_free (name);
-  	  
- 	  load_fallback_font (xfont);
+	  g_free (name);
+
+	  load_fallback_font (xfont);
 	}
     }
-  
+
   return xfont->xft_font;
 }
 
@@ -412,7 +416,7 @@ static FT_Face
 pango_xft_font_real_lock_face (PangoFcFont *font)
 {
   XftFont *xft_font = xft_font_get_font ((PangoFont *)font);
-  
+
   return XftLockFace (xft_font);
 }
 
@@ -420,7 +424,7 @@ static void
 pango_xft_font_real_unlock_face (PangoFcFont *font)
 {
   XftFont *xft_font = xft_font_get_font ((PangoFont *)font);
-  
+
   XftUnlockFace (xft_font);
 }
 
@@ -429,7 +433,7 @@ pango_xft_font_real_has_char (PangoFcFont *font,
 			      gunichar     wc)
 {
   XftFont *xft_font = xft_font_get_font ((PangoFont *)font);
-  
+
   return XftCharExists (NULL, xft_font, wc);
 }
 
@@ -438,7 +442,7 @@ pango_xft_font_real_get_glyph (PangoFcFont *font,
 			       gunichar     wc)
 {
   XftFont *xft_font = xft_font_get_font ((PangoFont *)font);
-  
+
   return XftCharIndex (NULL, xft_font, wc);
 }
 
@@ -446,7 +450,7 @@ static void
 pango_xft_font_real_shutdown (PangoFcFont *fcfont)
 {
   PangoXftFont *xfont = PANGO_XFT_FONT (fcfont);
-  
+
   if (xfont->xft_font)
     {
       Display *display;
@@ -463,21 +467,14 @@ pango_xft_font_real_shutdown (PangoFcFont *fcfont)
  *
  * Returns the XftFont of a font.
  *
- * Return value: the XftFont associated to @font.
+ * Return value: the XftFont associated to @font, or %NULL if @font is %NULL.
  **/
 XftFont *
 pango_xft_font_get_font (PangoFont *font)
 {
-  if (G_UNLIKELY (!PANGO_XFT_IS_FONT (font)))
-    {
-      if (!_pango_xft_warning_history.get_font)
-        {
-	  _pango_xft_warning_history.get_font = TRUE;
-	  g_warning ("pango_xft_font_get_font called with bad font, expect ugly output");
-	}
-      return NULL;
-    }
-  
+  if (G_UNLIKELY (!font))
+    return NULL;
+
   return xft_font_get_font (font);
 }
 
@@ -528,13 +525,13 @@ pango_xft_font_get_unknown_glyph (PangoFont *font,
  * pango_xft_font_lock_face:
  * @font: a #PangoFont.
  *
- * Gets the FreeType FT_Face associated with a font,
+ * Gets the FreeType <type>FT_Face</type> associated with a font,
  * This face will be kept around until you call
  * pango_xft_font_unlock_face().
  *
  * Use pango_fc_font_lock_face() instead.
  *
- * Return value: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType <type>FT_Face</type> associated with @font.
  *
  * Since: 1.2
  **/
@@ -569,11 +566,11 @@ pango_xft_font_unlock_face (PangoFont *font)
  * pango_xft_font_get_glyph:
  * @font: a #PangoFont for the Xft backend
  * @wc: Unicode codepoint to look up
- * 
+ *
  * Gets the glyph index for a given Unicode character
  * for @font. If you only want to determine
  * whether the font has the glyph, use pango_xft_font_has_char().
- * 
+ *
  * Use pango_fc_font_get_glyph() instead.
  *
  * Return value: the glyph index, or 0, if the Unicode
@@ -594,9 +591,9 @@ pango_xft_font_get_glyph (PangoFont *font,
  * pango_xft_font_has_char:
  * @font: a #PangoFont for the Xft backend
  * @wc: Unicode codepoint to look up
- * 
+ *
  * Determines whether @font has a glyph for the codepoint @wc.
- * 
+ *
  * Use pango_fc_font_has_char() instead.
  *
  * Return value: %TRUE if @font has the requested codepoint.
